package mixturedd.robotcontroler.remoter;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import java.io.IOException;

/**
 * MjpegView.java
 * Description :
 * <p>
 * Created by hp on 2016/12/13 23:04.
 * Copyright © 2016 hp. All rights reserved.
 */
public class MjpegSurfaceView extends SurfaceView implements SurfaceHolder.Callback {
    private static final String TAG = "MjpegView";

    public final static int POSITION_UPPER_LEFT = 9;
    public final static int POSITION_UPPER_RIGHT = 3;
    public final static int POSITION_LOWER_LEFT = 12;
    public final static int POSITION_LOWER_RIGHT = 6;

    public final static int SIZE_STANDARD = 1;
    public final static int SIZE_BEST_FIT = 4;
    public final static int SIZE_FULLSCREEN = 8;

    private MjpegViewThread thread;
    private MjpegInputStream mIn = null;
//    private Bitmap videoWarningBitmap;
    private boolean showFps = false;
    private boolean mRun = false;
    private boolean surfaceDone = false;
    private Paint overlayPaint;
    private int overlayTextColor;
    private int overlayBackgroundColor;
    private int ovlPos;
    private int dispWidth;
    private int dispHeight;
    private int displayMode;

    public class MjpegViewThread extends Thread {
        private final SurfaceHolder mSurfaceHolder;
        private int frameCounter = 0;
        private long start;
        private Bitmap ovl;

        public MjpegViewThread(SurfaceHolder surfaceHolder, Context context) {
            mSurfaceHolder = surfaceHolder;
        }

        private Rect destRect(int bitmapCachew, int bitmapCacheh) {
            int tempx;
            int tempy;
            if (displayMode == MjpegSurfaceView.SIZE_STANDARD) {
                tempx = (dispWidth / 2) - (bitmapCachew / 2);
                tempy = (dispHeight / 2) - (bitmapCacheh / 2);
                return new Rect(tempx, tempy, bitmapCachew + tempx, bitmapCacheh + tempy);
            }
            if (displayMode == MjpegSurfaceView.SIZE_BEST_FIT) {
                float bitmapCacheasp = (float) bitmapCachew / (float) bitmapCacheh;
                bitmapCachew = dispWidth;
                bitmapCacheh = (int) (dispWidth / bitmapCacheasp);
                if (bitmapCacheh > dispHeight) {
                    bitmapCacheh = dispHeight;
                    bitmapCachew = (int) (dispHeight * bitmapCacheasp);
                }
                tempx = (dispWidth / 2) - (bitmapCachew / 2);
                tempy = (dispHeight / 2) - (bitmapCacheh / 2);
                return new Rect(tempx, tempy, bitmapCachew + tempx, bitmapCacheh + tempy);
            }
            if (displayMode == MjpegSurfaceView.SIZE_FULLSCREEN) {
                return new Rect(0, 0, dispWidth, dispHeight);
            }
            return null;
        }

        public void setSurfaceSize(int width, int height) {
            synchronized (mSurfaceHolder) {
                dispWidth = width;
                dispHeight = height;
            }
        }

        private Bitmap makeFpsOverlay(Paint p, String text) {
            Rect b = new Rect();
            p.getTextBounds(text, 0, text.length(), b);
            int bwidth = b.width() + 2;
            int bheight = b.height() + 2;
            Bitmap bitmapCache = Bitmap.createBitmap(bwidth, bheight, Bitmap.Config.ARGB_8888);
            Canvas c = new Canvas(bitmapCache);
            p.setColor(overlayBackgroundColor);
            c.drawRect(0, 0, bwidth, bheight, p);
            p.setColor(overlayTextColor);
            c.drawText(text, -b.left + 1, (bheight / 2) - ((p.ascent() + p.descent()) / 2) + 1, p);
            return bitmapCache;
        }

        public void run() {
            start = System.currentTimeMillis();
            PorterDuffXfermode mode = new PorterDuffXfermode(PorterDuff.Mode.DST_OVER);
//            Bitmap bitmapCache;
            int width;
            int height;
            Rect destRect = null;
            Canvas c = null;
            Paint p = new Paint();
            String fps;
            Bitmap bitmapCache = null;
            while (mRun) {
                if (surfaceDone) {
                    try {
                        c = mSurfaceHolder.lockCanvas();
                        synchronized (mSurfaceHolder) {
                            try {
                                bitmapCache = mIn.readMjpegFrame();
                                if (bitmapCache != null){
                                    destRect = destRect(bitmapCache.getWidth(), bitmapCache.getHeight());
                                    //设置视频背景
                                    c.drawColor(Color.BLACK);
                                    if (destRect != null){
                                        c.drawBitmap(bitmapCache, null, destRect, p);
                                    }
                                }
                                //fps显示
                                if (showFps && destRect != null) {
                                    p.setXfermode(mode);
                                    if (ovl != null) {
                                        height = ((ovlPos & 1) == 1) ? destRect.top : destRect.bottom - ovl.getHeight();
                                        width = ((ovlPos & 8) == 8) ? destRect.left : destRect.right - ovl.getWidth();
                                        c.drawBitmap(ovl, width, height, null);
                                    }
                                    p.setXfermode(null);
                                    frameCounter++;
                                    if ((System.currentTimeMillis() - start) >= 1000) {
                                        fps = String.valueOf(frameCounter) + " fps";
                                        frameCounter = 0;
                                        start = System.currentTimeMillis();
                                        ovl = makeFpsOverlay(overlayPaint, fps);
                                    }
                                }
                            } catch (IOException e) {
                                e.getStackTrace();
                                Log.d(TAG, "catch IOException hit in run", e);
                            }
                        }
                    } finally {
                        if (c != null) {
                            mSurfaceHolder.unlockCanvasAndPost(c);
                        }
                    }
                }
            }
            if(bitmapCache != null && !bitmapCache.isRecycled()){
                bitmapCache.recycle();
                bitmapCache = null;
            }
        }
    }

    private void init(Context context) {
        SurfaceHolder holder = getHolder();
        holder.addCallback(this);
        thread = new MjpegViewThread(holder, context);
        setFocusable(true);
//        setBackgroundResource(R.drawable.video_bg);
//        videoWarningBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.video_warning);
        overlayPaint = new Paint();
        overlayPaint.setTextAlign(Paint.Align.LEFT);
        overlayPaint.setTextSize(12);
        overlayPaint.setTypeface(Typeface.DEFAULT);
        overlayTextColor = Color.WHITE;
        overlayBackgroundColor = Color.BLACK;
        ovlPos = MjpegSurfaceView.POSITION_LOWER_RIGHT;
        displayMode = MjpegSurfaceView.SIZE_STANDARD;
        dispWidth = getWidth();
        dispHeight = getHeight();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
    }

    public void startPlayback() {
        if (mIn != null) {
            mRun = true;
//            ThreadManager.getInstance().getThreadPoolExecutor().execute(thread);
            thread.start();
        }
    }

    public void stopPlayback() {
        mRun = false;
        boolean retry = true;
        while (retry) {
            try {
                thread.join();
                retry = false;
            } catch (InterruptedException e) {
                e.getStackTrace();
                Log.d(TAG, "catch IOException hit in stopPlayback", e);
            }
        }
    }

    public MjpegSurfaceView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context);
    }

    public MjpegSurfaceView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    public void surfaceChanged(SurfaceHolder holder, int f, int w, int h) {
        thread.setSurfaceSize(w, h);
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        surfaceDone = false;
/*        if(bitmapCache != null && !bitmapCache.isRecycled()){
            bitmapCache.recycle();
            bitmapCache = null;
        }
        ;*/
        stopPlayback();
    }

    public MjpegSurfaceView(Context context) {
        super(context);
        init(context);
    }

    public void surfaceCreated(SurfaceHolder holder) {
        surfaceDone = true;
    }

    public void showFps(boolean b) {
        showFps = b;
    }

    public void setSource(MjpegInputStream source) {
        mIn = source;
        startPlayback();
    }

    public void setOverlayPaint(Paint p) {
        overlayPaint = p;
    }

    public void setOverlayTextColor(int c) {
        overlayTextColor = c;
    }

    public void setOverlayBackgroundColor(int c) {
        overlayBackgroundColor = c;
    }

    public void setOverlayPosition(int p) {
        ovlPos = p;
    }

    public void setDisplayMode(int s) {
        displayMode = s;
    }

    public boolean isRun() {
        return mRun;
    }
}
